@@ -239,38 +239,16 @@ static const gchar *const g_utf8_skip = utf8_skip_data;
  * Return value: the length of the string in characters
  **/
 static glong
-g_utf8_strlen (const gchar * p, gssize max)
+g_utf8_strlen (const gchar * p)
 {
   glong len = 0;
-  const gchar *start = p;
-  g_return_val_if_fail (p != NULL || max == 0, 0);
 
-  if (max < 0)
-    {
-      while (*p)
-	{
-	  p = g_utf8_next_char (p);
-	  ++len;
-	}
-    }
-  else
-    {
-      if (max == 0 || !*p)
-	return 0;
+  g_return_val_if_fail (p != NULL, 0);
 
+  while (*p)
+    {
       p = g_utf8_next_char (p);
-
-      while (p - start < max && *p)
-	{
-	  ++len;
-	  p = g_utf8_next_char (p);
-	}
-
-      /* only do the last len increment if we got a complete
-       * char (don't count partial chars)
-       */
-      if (p - start <= max)
-	++len;
+      ++len;
     }
 
   return len;
@@ -699,22 +677,26 @@ find_decomposition (gunichar ch, gboolean compat)
 static gboolean
 combine_hangul (gunichar a, gunichar b, gunichar * result)
 {
-  gint LIndex = a - LBase;
-  gint SIndex = a - SBase;
-
-  gint VIndex = b - VBase;
-  gint TIndex = b - TBase;
-
-  if (0 <= LIndex && LIndex < LCount && 0 <= VIndex && VIndex < VCount)
+  if (a >= LBase && a < LCount + LBase && b >= VBase && b < VCount + VBase)
     {
+      gint LIndex = a - LBase;
+      gint VIndex = b - VBase;
+
       *result = SBase + (LIndex * VCount + VIndex) * TCount;
       return TRUE;
     }
-  else if (0 <= SIndex && SIndex < SCount && (SIndex % TCount) == 0
-	   && 0 < TIndex && TIndex < TCount)
+
+  if (a >= SBase && a < SCount + SBase && b > TBase && b < TCount + TBase)
     {
-      *result = a + TIndex;
-      return TRUE;
+      gint SIndex = a - SBase;
+
+		if ((SIndex % TCount) == 0)
+        {
+          gint TIndex = b - TBase;
+
+          *result = a + TIndex;
+          return TRUE;
+        }
     }
 
   return FALSE;
@@ -811,7 +793,7 @@ _g_utf8_normalize_wc (const gchar * str, gssize max_len, GNormalizeMode mode)
 	  decomp = find_decomposition (wc, do_compat);
 
 	  if (decomp)
-	    n_wc += g_utf8_strlen (decomp, -1);
+	    n_wc += g_utf8_strlen (decomp);
 	  else
 	    n_wc++;
 	}
@@ -872,7 +854,7 @@ _g_utf8_normalize_wc (const gchar * str, gssize max_len, GNormalizeMode mode)
     {
       g_unicode_canonical_ordering (wc_buffer + last_start,
 				    n_wc - last_start);
-      // dead assignment: last_start = n_wc;
+      /* dead assignment: last_start = n_wc; */
     }
 
   wc_buffer[n_wc] = 0;
@@ -959,9 +941,11 @@ static gchar *
 g_utf8_normalize (const gchar * str, gssize len, GNormalizeMode mode)
 {
   gunichar *result_wc = _g_utf8_normalize_wc (str, len, mode);
-  gchar *result;
+  gchar *result = NULL;
+
+  if (result_wc)
+    result = g_ucs4_to_utf8 (result_wc, -1, NULL, NULL);
 
-  result = g_ucs4_to_utf8 (result_wc, -1, NULL, NULL);
   g_free (result_wc);
 
   return result;
@@ -1099,6 +1083,7 @@ stringprep_utf8_nfkc_normalize (const char *str, ssize_t len)
   return g_utf8_normalize (str, len, G_NORMALIZE_NFKC);
 }
 
+#include <stdio.h>
 /**
  * stringprep_ucs4_nfkc_normalize:
  * @str: a Unicode string.
@@ -1117,6 +1102,9 @@ stringprep_ucs4_nfkc_normalize (const uint32_t * str, ssize_t len)
   uint32_t *result_wc;
 
   p = stringprep_ucs4_to_utf8 (str, len, 0, 0);
+  if (!p)
+    return NULL;
+
   result_wc = _g_utf8_normalize_wc (p, -1, G_NORMALIZE_NFKC);
   free (p);
 
